### Build ###
target/
build/
bin/
dist/
db/
!.mvn/wrapper/maven-wrapper.jar

### Gradle ###
.gradle/

## VSCode ##
.vscode

### Eclipse ###
.apt_generated
.classpath
.factorypath
.project
.settings
.springBeans

### IntelliJ IDEA ###
.idea
*.iws
*.iml
*.ipr

### NetBeans ###
nbproject/private/
nbbuild/
nbdist/
.nb-gradle/

# Node and Vue
node_modules
/dist
*.suo
*.ntvs*
*.njsproj
*.sln
*.sw*

# local env files
.env.local
.env.*.local

# Compiled source #
###################
*.com
*.class
*.dll
*.exe
*.o
*.so
*.pyc
*.pid

# Packages #
############
# it's better to unpack these files and commit the raw source
# git has its own built in compression methods
*.7z
*.dmg
*.gz
*.iso
*.jar
*.rar
*.tar
*.zip

# Logs and databases #
######################
*.log
log/

# OS generated files #
######################
.DS_Store
.DS_Store?
]con[
._*
.Spotlight-V100
.Trashes
ehthumbs.db
Thumbs.db
Desktop.ini
