# _*_ coding: utf-8 _*_
#
# hspylib v0.11.121
#
# Package: main.hspylib.addons.widman
"""Package initialization."""

__all__ = [
  'widget', 
  'widget_entry', 
  'widgets', 
  'widman'
]
__version__ = '0.11.121'
