APP_NAME v{} - APP_DESC.

Usage: APP_NAME [options] <arguments>

  Options:
    -v  |  --version  : Display current program version.
    -h  |     --help  : Display this help message.

  Arguments:
    argument: TODO
